/*
   @file chemical_formula_parser.cc
   @brief Parse chemical (molecular) formulas
   @author (C) Copyright 2010 Paolo Greppi libpf.com
   @date 20100923
   @version 0.1

   no warranties whatsoever
   distribute freely and free of charge citing this:
   https://libpf.com/posts/2010-09-23-a-chemical-formula-parser-for-boostspirit
*/

#include <iostream>
#include <map>
#include <string>

#include <boost/spirit/include/classic_core.hpp>

std::string s_;
int n_;

void prefetch_s(char const*start, char const*end) {
  s_.assign(start, end-start);
  n_ = 1;
}

void prefetch_n(int n) {
  n_ = n;
}

std::map<std::string, int> atomcount;

void fetch_s(char const*start, char const*end) {
  s_.assign(start, end-start);
  atomcount[s_] += 1;
}

void fetch(char const*, char const*) {
  atomcount[s_] += n_;
}

bool parse_formula(char const* str) {
  atomcount.clear();
  return parse(str,
               *(
                  (
                     (boost::spirit::classic::range_p('A','Z') >> !boost::spirit::classic::range_p('a','z'))[&prefetch_s] >> 
                       boost::spirit::classic::int_p[&prefetch_n]
                  )[&fetch]
                 |
                   (boost::spirit::classic::range_p('A','Z') >> !boost::spirit::classic::range_p('a','z'))[&fetch_s]
                )
               ,
               boost::spirit::classic::space_p).full;
}

int main(int argc, char *argv []) {
  std::cout << "/////////////////////////////////////////////////////////\n\n";
  std::cout << "\t\tA chemical formula parser for Spirit...\n\n";
  std::cout << "\t\t(C) Copyright 2010 Paolo Greppi 3iP\n";
  std::cout << "\t\tdate 20100923 version 0.1\n\n";
  std::cout << "\t\tno warranties whatsoever\n";
  std::cout << "\t\tdistribute freely and free of charge citing this:\n";
  std::cout << "\t\thttps://libpf.com/posts/2010-09-23-a-chemical-formula-parser-for-boostspirit\n";
  std::cout << "/////////////////////////////////////////////////////////\n\n";

  std::cout << "Give me a chemical formula.\n";
  std::cout << "Type [q or Q] to quit\n\n";

  std::string str;
  while (std::getline(std::cin, str))
  {
    if (str.empty() || str[0] == 'q' || str[0] == 'Q')
      break;
    if (parse_formula(str.c_str())) {
      std::cout << "-------------------------\n";
      std::cout << "Parsing succeeded\n";
      std::cout << str << " Parses OK: " << std::endl;

      for (std::map<std::string, int>::const_iterator i = atomcount.begin(); i != atomcount.end(); ++i) {
        std::cout <<  i->first << " : " << i->second << " " << std::endl;
      }
      std::cout << "-------------------------\n";
    } else {
      std::cout << "-------------------------\n";
      std::cout << "Parsing failed\n";
      std::cout << "-------------------------\n";
    }
  }
  std::cout << "Bye... :-) \n\n";
  return 0;
} // main
